/* 
** attributes.c: Handles the attributes of widgets
** copyright: (c) 2003 by Lszl Pere
** email: pipas@linux.pte.hu
**
** This program is free software; you can redistribute it and/or 
** modify  it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of
** the License, or (at your option) any later version.
*/

#include "attributes.h"

AttributeSet *attributeset_new(void)
{
	AttributeSet *new_set;
	int n;

	new_set = pip_malloc(sizeof(AttributeSet), __func__);
	new_set->n_attr = ATTRNUMBER;
	new_set->attr = pip_malloc(sizeof(GList *) * ATTRNUMBER, __func__);
	new_set->_pointer = pip_malloc(sizeof(int) * ATTRNUMBER, __func__);
	for (n = 0; n < ATTRNUMBER; ++n) {
		new_set->attr[n] = NULL;
		new_set->_pointer[n] = NULL;
	}
	return (new_set);
}

gboolean attributeset_is_avail(AttributeSet * set, int attribute)
{
	if ((set == NULL) | (attribute < 0) | (attribute >= ATTRNUMBER))
		return (FALSE);

	if (set->attr[attribute] == NULL)
		return (FALSE);
	else
		return (TRUE);
}

gboolean attributeset_cmp_left(AttributeSet * set, int attribute,
			       const char *str)
{
	if (!attributeset_is_avail(set, attribute))
		return (FALSE);
	if (strncasecmp(set->attr[attribute]->data, str, strlen(str)) == 0)
		return (TRUE);
	else
		return (FALSE);
}

char *attributeset_get_first(AttributeSet * set, int attribute)
{
	if (set == NULL || attribute >= set->n_attr ||
	    set->attr[attribute] == NULL) {
#ifdef DEBUG
		fprintf(stderr, "%s(): Warning, first argument is NULL.\n",
			__func__);
		fflush(stderr);
#endif
		return (NULL);
	}

	set->_pointer[attribute] = set->attr[attribute];
	return (set->_pointer[attribute]->data);
}

char *attributeset_get_next(AttributeSet * set, int attribute)
{
	if ((set == NULL) | (attribute >= set->n_attr) ||
	    (g_list_next(set->_pointer[attribute])) == NULL)
		return (NULL);
	set->_pointer[attribute] = g_list_next(set->_pointer[attribute]);
	return (set->_pointer[attribute]->data);
}

GList *attributeset_get_whole_list(AttributeSet * set, int attribute)
{
	if (!attributeset_is_avail(set, attribute)) {
		fprintf(stderr,
			"%s(): Warning: getting uninitialized GList.\n",
			__func__);
		fflush(stderr);
		return (NULL);
	}
	return (set->attr[attribute]);
}

char *attributeset_set_if_unset(AttributeSet * set, int attribute,
				const char *s)
{
	if (set == NULL || attribute >= set->n_attr)
		return (NULL);

	if (set->attr[attribute] != NULL)
		return (set->attr[attribute]->data);

	return (attributeset_insert(set, attribute, s));
}


char *attributeset_insert(AttributeSet * set, int attribute, const char *s)
{
	char *value;
	/* Some error checking needed. 
	 */
	if (s == NULL)
		s = "(null)";
	if (set == NULL)
		set = attributeset_new();
	/* FIXME: We should allocate more memory in the future. */
	if (attribute >= set->n_attr || attribute < 0) {
		fprintf(stderr, "%s(): Invalid attribute: %d\n", __func__,
			attribute);
		return (NULL);
	}

	/* We make a copy of the string and insert into the list. The 
	 * return value is the newly allocated string.
	 */
	value = strdup(s);
	if (value == NULL) {
		fprintf(stderr, "%s(): Could not allocate memory.\n",
			__func__);
	}
	set->attr[attribute] = g_list_append(set->attr[attribute], value);
	return (value);
}

int attributeset_clean(AttributeSet * set)
{
	int n;
	if (set == NULL)
		return (-1);

	for (n = 0; n < set->n_attr; ++n)
		if (set->attr[n] != NULL) {
			//g_list_free( set->attr[n] );
			set->attr[n] = NULL;
		}

	return (0);
}

