/*
** automaton.c: An automaton executing the program, creating widgets. 
** copyright: (c) 2003 by Lszl Pere
** email: pipas@linux.pte.hu
**
** This program is free software; you can redistribute it and/or 
** modify  it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of
** the License, or (at your option) any later version.
**
** $Id: automaton.c,v 1.2 2004/11/18 16:45:52 root Exp pipas $
** $Log: automaton.c,v $
** Revision 1.2  2004/11/18 16:45:52  root
**   The button now can have a pixmap and a label simultaneously.
**
** Revision 1.1  2004/11/16 20:45:22  root
** Initial revision
**
*/
#include "automaton.h"
#include "gtkdialog.h"
#include "stringman.h"
#include "actions.h"	//GN
//GN ADDITIONS
#ifndef USE_GTK2
GtkWidget *box1;
#define GTK_STOCK_CANCEL "gtk-cancel.xpm"    
#define GTK_STOCK_HELP   "gtk-help.xpm"
#define GTK_STOCK_NO     "gtk-no.xpm"
#define GTK_STOCK_OK     "gtk-ok.xpm"
#define GTK_STOCK_YES    "gtk-yes.xpm" 
#endif



/* Create a new hbox with an image and a label packed into it and return the box. */

GtkWidget *xpm_label_box( GtkWidget *parent,
                          gchar     *xpm_filename,
                          gchar     *label_text )
{
    GtkWidget *box1;
    GtkWidget *label;
    GtkWidget *pixmapwid;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    GdkColormap *colormap; 

    /* Create box for xpm and label */
    box1 = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (box1), 2);

    colormap = gtk_widget_get_colormap (parent);
	pixmap = gdk_pixmap_colormap_create_from_xpm (NULL,
								colormap, &mask, NULL, xpm_filename);
	
    pixmapwid = gtk_pixmap_new (pixmap, mask);

    /* Create a label for the button */
    label = gtk_label_new (label_text);
    //gtk_misc_set_padding (label, 0, 0);
    /* Pack the pixmap and label into the box */
    gtk_box_pack_start (GTK_BOX (box1), pixmapwid, FALSE, FALSE, 3);

    gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 3);

    gtk_widget_show(pixmapwid);
    gtk_widget_show(label);

    return(box1);
}


//GN END

#undef TOOLTIPS

extern char *includefile;
extern char *program_name;
char *wtitle;

instruction *program = NULL;
int instruction_counter = 0;	/* The first available memory cell. */
size_t memory_counter = 0;	/* The size of program memory.     */
GtkWidget *window = NULL;	/* The actual window.              */


gint delete(GtkWidget * widget, GtkWidget * event, gpointer data)
{
	gtk_main_quit();
	printf("EXIT=abort\n");
	exit(EXIT_SUCCESS);
}

void table_selection_made(GtkWidget * clist, gint row, gint column,
			  GdkEventButton * event, gpointer data)
{
	if (data == NULL)
		return;
#ifdef DEBUG
	fprintf(stderr, "%s(): Selected %d, %d\n", __func__, column, row);
	fflush(stderr);
#endif
	variables_set_row_column(data, row, column);
}

void table_selected(GtkWidget * clist, gint row, gint column,
		    GdkEventButton * event, gpointer data)
{

#ifdef DEBUG
	fprintf(stderr, "%s(): %s\n", __func__, data);
	fflush(stderr);
#endif

	if (data == NULL) {
		fprintf(stderr, "%s(): data is NULL.\n", __func__);
		fflush(stderr);
		return;
	}

	if (strncasecmp(data, "refresh:", 8) == 0) {
		variables_export_all();
		variables_refresh(&((char *) data)[8]);
		return;
	}

	/*
	 ** If there is no known prefix at the beginning of the command
	 ** string, we start it as a shell command.
	 */
	action_shellcommand(clist, data);
}

void entry_inserted(GtkWidget * entry, gpointer str)
{
	button_pressed(entry, str);
}

void button_toggled(GtkToggleButton * button, gpointer str)
{
	/*
	 ** First we check for "if true" and "if false" prefixes.
	 */
	if (strncasecmp(str, "if true ", 8) == 0) {
		if (gtk_toggle_button_get_active
		    (GTK_TOGGLE_BUTTON(button)))
			button_pressed((GtkWidget *) button,
				       &((char *) str)[8]);
		return;
	}
	if (strncasecmp(str, "if false ", 9) == 0) {
		if (!gtk_toggle_button_get_active
		    (GTK_TOGGLE_BUTTON(button)))
			button_pressed((GtkWidget *) button,
				       &((char *) str)[9]);
		return;
	}

	/*
	 ** Otherwise we just call the function we used to call when a
	 ** button is pressed.
	 */
	button_pressed((GtkWidget *) button, str);
	return;
}

void list_selection_changed(GtkWidget * list, gpointer str)
{
	button_pressed(list, str);
	return;
}


/*
** This function is called when a button is pressed. The first
** argument is a pointer to the window, the second is the command
** string.
*/
void button_pressed(GtkWidget * button, gpointer str)
{
#ifdef DEBUG
	fprintf(stderr, "%s(): action: '%s'\n", __func__, str);
	fflush(stderr);
#endif

	//if (strncasecmp(str, "Closewindow:", 12) == 0) {
	//	action_closewindow(button, &((char *) str)[12]);
	//	return;
	//}
	if( strncasecmp(str, "Closewindow:", 11)==0 ){
		gdk_window_destroy( gtk_widget_get_parent_window(button) );
		return;
	} 

	if (strncasecmp(str, "Launch:", 7) == 0) {
		action_launchwindow(button, &((char *) str)[7]);
		return;
	}

	if (strncasecmp(str, "Enable:", 7) == 0) {
		action_enable(button, &((char *) str)[7]);
		return;
	}
	if (strncasecmp(str, "Disable:", 8) == 0) {
		action_disable(button, &((char *) str)[8]);
		return;
	}

	if (strncasecmp(str, "exit", 4) == 0) {
		/*
		 ** This function will not return, it exits the
		 ** application.
		 */
		action_exitprogram(button, &((char *) str)[4]);
		return;
	}

	if (strncasecmp(str, "Refresh:", 8) == 0) {
		variables_export_all();
		action_refreshwidget(button, &((char *) str)[8]);
		return;
	}

	if (strncasecmp(str, "Save:", 5) == 0) {
		/* FIXME: is it working? */
		action_savewidget(button, &((char *) str)[5]);
		return;
	}

	if (strncasecmp(str, "fileselect:", 11) == 0) {
		action_fileselect(button, &((char *) str)[11]);
		return;
	}

	if (strncasecmp(str, "clear:", 6) == 0) {
		action_clearwidget(button, &((char *) str)[6]);
		return;
	}

	if (strncasecmp(str, "removeselected:", 15) == 0) {
		action_removeselected(button, &((char *) str)[15]);
		return;
	}

	if (strncasecmp(str, "append:", 6) == 0) {
		action_append(button, &((char *) str)[7]);
		return;
	}


	/*
	 ** If there is no known prefix at the beginning of the command
	 ** string, we start it as a shell command.
	 */
	action_shellcommand(button, str);
	return;
}

/* 
** We get the string between <wtitle> and </wtitle>, and give it for
** gtk_window_set_title() function in run_program. If the string is NULL
** we set the title to MAIN_DIALOG.
*/
void window_title(char *string)
{
	wtitle = string;
}

/*
** This function will run the program in memory.
*/
void run_program()
{
	int pc;
	#ifdef USE_GTK2 
	int q;
	#endif
	stackelement s;
	AttributeSet *Attr;
	
	/*
	 ** First we create a new window which will be the parent of
	 ** all widgets.
	 */
#ifdef EXECUTION_DEBUG
	fprintf(stderr, "%s(): program starting.\n", __func__);
	fflush(stderr);
#endif

	Attr = attributeset_new();
	
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	if (wtitle == NULL)
		wtitle = "gtkdialog";

	gtk_window_set_title(GTK_WINDOW(window), wtitle);
	gtk_container_set_border_width(GTK_CONTAINER(window), 5);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			   GTK_SIGNAL_FUNC(delete), NULL);

	variables_new_with_widget(Attr, window, WIDGET_WINDOW);
	variables_set_parent(program_name, window);
	
	for (pc = 0; pc < instruction_counter; ++pc)
		instruction_execute(program[pc]);
#ifdef EXECUTION_DEBUG
	fprintf(stderr, "%s(): program ended.\n", __func__);
	fflush(stderr);
#endif

	s = pop();

	gtk_container_add(GTK_CONTAINER(window), s.widgets[0]);
#ifdef EXECUTION_DEBUG
	fprintf(stderr, "%s(): window: %p.\n", __func__, window);
	fflush(stderr);
#endif
	gtk_widget_show_all(window);

	//
	// Now, all of the widgets are realized, so we can initialize
	// them as needed.
	//
	variables_initialize_all();	
	
	instruction_counter = 0;
	charsreaded = 0;
	gtk_main();
}

void print_token(token Token)
{
	int Instr_Code;
	int Widget_Type;
	int Attribute_Number;
	Instr_Code = Token & COMMAND;
	Widget_Type = Token & WIDGET_TYPE;
	Attribute_Number = Token & ATTRIBUTE;

	switch (Instr_Code) {
		case PUSH:
			printf("   %s(): instruction PUSH ", __func__);
			break;
		case SUM:
			printf("   %s(): instruction SUM ", __func__);
			break;
		case SET:
			printf("   %s(): instruction SET ", __func__);
			break;
		default:
			printf("   %s(): unknown instruction: '%d' ", __func__, 
					Instr_Code);
			break;
	}

	switch (Widget_Type) {
	case WIDGET_LABEL:
		printf("LABEL");
		break;
	case WIDGET_ENTRY:
		printf("ENTRY");
		break;
	case WIDGET_EDIT:
		printf("EDIT");
		break;
	case WIDGET_BUTTON:
		printf("BUTTON");
		break;
	case WIDGET_CHECKBOX:
		printf("CHECKBOX");
		break;
	case WIDGET_RADIO:
		printf("RADIO");
		break;
	case WIDGET_LIST:
		printf("LIST");
		break;
	case WIDGET_TABLE:
		printf("TABLE");
		break;
	case WIDGET_COMBO:
		printf("COMBO");
		break;
	case WIDGET_SCROLLEDW:
		printf("SCROLLEDW");
		break;
	case WIDGET_VBOX:
		printf("VBOX");
		break;
	case WIDGET_HBOX:
		printf("HBOX");
		break;
	case WIDGET_FRAME:
		printf("FRAME");
		break;
	case WIDGET_PIXMAP:
		printf("PIXMAP");
		break;
	default:
		printf("Widget (%d)", Widget_Type);
	}
	printf("\n");
	fflush(stdout);
}


int instruction_execute(instruction command)
{
	token Token;
	char *Argument;
	int Instr_Code;
	int Widget_Type;
	int Attribute_Number;
	static AttributeSet *Attr = NULL;

	Token = command.command;
	Argument = command.argument;

	Instr_Code = Token & COMMAND;
	Widget_Type = Token & WIDGET_TYPE;
	Attribute_Number = Token & ATTRIBUTE;
#ifdef EXECUTION_DEBUG
	fprintf(stderr,
		"%s(): Executing token %08x with argument '%s'.\n",
		__func__, Token, Argument);
	fprintf(stderr,
		"   inst:%08x, widgt:%08x, argn:%08x\n",
		Instr_Code, Widget_Type, Attribute_Number);
	print_token(Token);
	fflush(stderr);
#endif

	if (Attr == NULL)
		Attr = attributeset_new();

	switch (Instr_Code) {
	case PUSH:
		/* These instructions will push widgets into the stack, some
		 ** of them will remove elements from it.
		 */
		instruction_execute_push(Token, Attr);

		variables_refresh(attributeset_get_first
				  (Attr, ATTR_VARIABLE));
		Attr = NULL;
		break;

	case SET:
		/* 
		 ** SET instructions stores strings in the attributes.
		 */
		attributeset_insert(Attr, Attribute_Number, Argument);
		break;
	case SUM:
		/*
		 ** This instruction will add the to uppermost stack element and
		 ** push the result to the stack, so the upper element will 
		 ** contain all the widgets.
		 */
		push(_sum(pop(), pop()));
		break;
		break;
	default:
		fprintf(stderr, "%s(): Unknown instruction.\n", __func__);
	}
	return (0);
}

static void
menu_callback (GtkWidget *menu_item, char *data)
{
#ifdef DEBUG
	fprintf(stderr, "%s(): Start.\n", __func__);
	fflush(stderr);
#endif
	button_pressed(menu_item, data);
}


static GtkWidget *create_menuitem(AttributeSet *Attr){
	GtkWidget *menu_item;
	// Example: GTK_STOCK_OPEN
	if (strncmp("gtk-", attributeset_get_first(Attr, ATTR_LABEL), 4) == 0){
		#ifdef USE_GTK2 		
			menu_item = gtk_image_menu_item_new_from_stock(
				attributeset_get_first(Attr, ATTR_LABEL), NULL);
		#else
		
		//GN added below FIXME below if possible - need images!
		menu_item = gtk_menu_item_new_with_label (
				attributeset_get_first(Attr, ATTR_LABEL));
		#endif
		
	} else {
		menu_item = gtk_menu_item_new_with_label (
				attributeset_get_first(Attr, ATTR_LABEL));
	}
	
	if (attributeset_is_avail(Attr, ATTR_ACTION))
		gtk_signal_connect(GTK_OBJECT(menu_item),
		"activate",
		menu_callback, attributeset_get_first(Attr, ATTR_ACTION));
	
	
	return menu_item;
}

static
GtkWidget *create_menu(AttributeSet *Attr, stackelement items){
	int n;
	GtkWidget *menu;
	GtkWidget *root_menu;
	GtkWidget *menu_items;
	
	menu = gtk_menu_new();
	for (n = 0; n < items.nwidgets; ++n){
		menu_items = items.widgets[n];
        	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
	}
	
	root_menu = gtk_menu_item_new_with_label (
			attributeset_get_first(Attr, ATTR_LABEL));
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (root_menu), menu);
	
	return root_menu;
}

static
GtkWidget *create_menubar(AttributeSet * Attr, stackelement menus)
{
	int n;
	GtkWidget *menu_bar;
	GtkWidget *root_menu;
	
	menu_bar = gtk_menu_bar_new ();
	for (n = 0; n < menus.nwidgets; ++n){
		root_menu = menus.widgets[n];
		gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), root_menu);
	}
	
	return menu_bar;
}

static
GtkWidget *create_label(AttributeSet * Attr)
{
	GtkWidget *Label;
#if 0
	char *text = NULL;
	if (attributeset_is_avail(Attr, ATTR_LABEL))
		text = attributeset_get_first(Attr, ATTR_LABEL);
	if (text == NULL &&
			attributeset_is_avail(Attr
#endif
	/* 
	 ** Setting up the default values for the new label.
	 */
	attributeset_set_if_unset(Attr, ATTR_LABEL,
				  "(unnamed label)");
	/*
	 ** Creating the widget, and pushing it into the stack.
	 */
	Label = gtk_label_new(attributeset_get_first
			  (Attr, ATTR_LABEL));
	
	if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled"))
		gtk_widget_set_sensitive(Label, FALSE);

	return Label;
}

static gboolean widget_moved(GtkWidget *widget, GdkEvent *event, gpointer user_data) {
	#ifdef USE_GTK2 
		GtkRequisition size;
	#endif
		GtkWindow *window = user_data;
		GdkEventType type;
		GdkEventConfigure  *configure;
		type = event->type;
	
	if (type == 2)
		return FALSE;
	#if 0
		fprintf(stderr, "%s(): event->type: %d mask %x.\n", __func__, 
			type,
			gtk_widget_get_events(widget));
	#endif	
  	if (type == GDK_EXPOSE ){
		return FALSE;
	}
	
	if (type == GDK_CONFIGURE){
		configure = (GdkEventConfigure *)event;
	#if 0
		printf("   ->x = %d y = %d dx = %d dy = %d\n", 
			configure->x,
			configure->y,
			configure->width,
			configure->height);
	#endif
		//gtk_widget_set_usize(window, 
		//		configure->width -20,
		//		configure->height);
		gtk_widget_set_uposition(GTK_WINDOW(window), 
				configure->x,
				configure->y);
	}
	
	fflush(stderr);

	return FALSE;
}

static GtkWidget *create_gvim (AttributeSet * Attr) {
	GtkWidget *Window = gtk_window_new(GTK_WINDOW_POPUP);
	GtkWidget *socket = gtk_socket_new();
	GtkWidget *fake = gtk_drawing_area_new();
	gtk_widget_set_events(fake, GDK_ALL_EVENTS_MASK);
	gtk_widget_set_extension_events (fake, GDK_EXTENSION_EVENTS_ALL);
	char socket_id[32];

	//gtk_window_set_transient_for(Window, window);
	gtk_widget_set_usize(Window, 350, 150);
	gtk_widget_set_usize(fake, 350, 150);
	
	gtk_container_add (GTK_CONTAINER (Window), socket);
	gtk_widget_show (socket);
	gtk_widget_show (Window);
	gtk_widget_realize (socket);
	//GN sprintf(socket_id, "%d", gtk_socket_get_id (socket));
	attributeset_set_if_unset(Attr, ATTR_SOCKET, socket_id); 
	//
	// 
	//
	//gtk_widget_add_events(fake, GDK_ALL_EVENTS_MASK);
	//g_signal_connect(G_OBJECT(fake), "configure-event",
	//		 widget_moved, Window);
	gtk_signal_connect(GTK_OBJECT(window), "configure-event",
			 widget_moved, Window);
	//g_signal_connect(G_OBJECT(window), "event",
	//		 widget_moved, Window);
	return fake;
}

#ifdef USE_GTK2 
static GtkTreeStore *create_store_from_attr(AttributeSet *Attr){
	GtkTreeStore *store;
	GType types[32];
	GtkTreeIter iter[32];
	list_t *lines = NULL;
	list_t *lines_before = NULL;
	int column_number;
	char *line;
	int level;
	
	list_t *columns;
	int n, q;
	
	if (!attributeset_is_avail(Attr, ATTR_LABEL) ||
	    !attributeset_is_avail(Attr, ATTR_ITEM)) 
		return NULL;
	
	columns = linecutter(strdup(attributeset_get_first(Attr, ATTR_LABEL)), '|');
	column_number = columns->n_lines;
	
	/*
	 * Creating a store
	 */

	for (n = 0; n < 32; ++n){
		types[n] = G_TYPE_STRING;
	}
	store = gtk_tree_store_newv(column_number, types);  
	
	/*
	 * Inserting lines
	 */
	
	line = attributeset_get_first(Attr, ATTR_ITEM);
	level = 0;
	while (line != NULL){
		lines = linecutter(strdup(line), '|'); 
		
		if (lines_before == NULL ||
			strcmp(lines->line[0], lines_before->line[0]) != 0){
			gtk_tree_store_append (store, &iter[0], NULL);
			level = 0;
			goto go;
		}
		
		if (level == 0 && 
				strcmp(lines->line[0], lines_before->line[0]) == 0){
			gtk_tree_store_append (store, &iter[1], &iter[0]);
			level = 1;
			goto go;
		}
		
		if (level < column_number){
			for(q = 1; q <= level; ++q)
				if (strcmp(lines->line[q], lines_before->line[q]) != 0){
					gtk_tree_store_append (store, &iter[q], &iter[q-1]);
					level = q;
					goto go;
			}
			gtk_tree_store_append (store, &iter[level+1], &iter[level]);
			++level;
		}
		
					
go:
		if (lines->n_lines == 2)
			gtk_tree_store_set (store, &iter[level],
                    			0, lines->line[0],
                    			1, lines->line[1],
                    			-1);
		if (lines->n_lines == 3){
			//if (level == 0)
				gtk_tree_store_set (store, &iter[level],
                    				0, lines->line[0],
                    				1, lines->line[1],
                    				2, lines->line[2],
                    				-1);
#if 0
			if (level == 1)
				gtk_tree_store_set (store, &iter[level],
                    				1, lines->line[1],
                    				2, lines->line[2],
                    				-1);
			if (level == 2)
				gtk_tree_store_set (store, &iter[level],
                    				2, lines->line[2],
                    				-1);
#endif
		}
		if (lines->n_lines == 4)
			gtk_tree_store_set (store, &iter[level],
                    			0, lines->line[0],
                    			1, lines->line[1],
                    			2, lines->line[2],
                    			3, lines->line[3],
                    			-1);
		line = attributeset_get_next(Attr, ATTR_ITEM);
		lines_before = lines;
	}
	return store;
}
#endif

//GN add

/* Note that this is never called */
//GNstatic void cb_unselect_child( GtkWidget *root_tree,
//GN                               GtkWidget *child,
//GN                               GtkWidget *subtree )
//GN{
//GN  g_print ("unselect_child called for root tree %p, subtree %p, child %p\n",
//GN           root_tree, subtree, child);
//GN}

//GNstatic void cb_select_child (GtkWidget *root_tree, GtkWidget *child,
//GN                             GtkWidget *subtree)
//GN{
//GN  g_print ("select_child called for root tree %p, subtree %p, child %p\n",
//GN           root_tree, subtree, child);
//GN}

//GNstatic void cb_selection_changed( GtkWidget *tree )
//GN{
//GN  GList *i;
//GN  
//GN  g_print ("selection_change called for tree %p\n", tree);
//GN  g_print ("selected objects are:\n");
//GN
//GN  i = GTK_TREE_SELECTION(tree);
//GN  while (i){
//GN    gchar *name;
//GN    GtkLabel *label;
//GN    GtkWidget *item;
//GN
//GN    /* Get a GtkWidget pointer from the list node */
//GN    item = GTK_WIDGET (i->data);
//GN    label = GTK_LABEL (GTK_BIN (item)->child);
//GN    gtk_label_get (label, &name);
//GN    g_print ("\t%s on level %d\n", name, GTK_TREE
//GN             (item->parent)->level);
//GN    i = i->next;
//GN  }
//GN}


//GN add end

static GtkWidget *create_tree(AttributeSet * Attr) {
	list_t *columns;
	int n;

	if (attributeset_is_avail(Attr, ATTR_LABEL)) {
		columns = linecutter(strdup(attributeset_get_first(Attr,
		ATTR_LABEL)), '|');
	}
	
	#ifndef USE_GTK2 
	/* a generic toplevel window */
  	//		window = gtk_window_new(GTK_WINDOW_POPUP);
  	//		gtk_signal_connect (GTK_OBJECT(window), "delete_event",
    //                  GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  	//		gtk_container_set_border_width (GTK_CONTAINER(window), 5);
	
	#endif
	
	GtkWidget *ScrolledWindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
		(ScrolledWindow),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_ALWAYS);

	if (attributeset_is_avail(Attr, ATTR_HEIGHT) &&
	    attributeset_is_avail(Attr, ATTR_WIDTH))
		gtk_widget_set_usize(ScrolledWindow, 
				atoi(attributeset_get_first(Attr, ATTR_WIDTH)),
				atoi(attributeset_get_first(Attr, ATTR_HEIGHT)));
	else
		gtk_widget_set_usize(ScrolledWindow, 400, 200);
		
		#ifdef USE_GTK2 
			GtkTreeStore *store = create_store_from_attr(Attr);
			tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
			GtkCellRenderer *renderer;
			GtkTreeViewColumn *column;
		#else
			 	
		 	//gtk_container_add (GTK_CONTAINER(window), scrolled_win);
  			//gtk_widget_show (scrolled_win);
		
//GN			GtkWidget *tree;
//GN			tree = gtk_tree_new();
//GN			g_print ("root tree is %p\n", tree);
//GN			gtk_signal_connect (GTK_OBJECT(tree), "select_child",
//GN                      GTK_SIGNAL_FUNC(cb_select_child), tree);
//GN 			gtk_signal_connect (GTK_OBJECT(tree), "unselect_child",
//GN                      GTK_SIGNAL_FUNC(cb_unselect_child), tree);
//GN 			gtk_signal_connect (GTK_OBJECT(tree), "selection_changed",
//GN                      GTK_SIGNAL_FUNC(cb_selection_changed), tree);
		#endif

	for (n = 0; n < columns->n_lines; ++n){
		//GN renderer = gtk_cell_renderer_text_new ();
		/*GN column = gtk_tree_view_column_new_with_attributes (
				columns->line[n],
                                renderer,
                                "text", n,
                                NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
		GN*/
	}
	
	//gtk_container_add(GTK_CONTAINER(ScrolledWindow), tree);

	return ScrolledWindow;
}

#define TYPE_NOTHING 0
#define TYPE_LAB 1
#define TYPE_PIX 2
#define TYPE_LABPIX 3

static GtkWidget *create_button(AttributeSet * Attr) {
	GtkWidget *Button = NULL;
	#ifdef USE_GTK2
		GtkWidget *Icon = NULL;
	#endif
	GtkWidget *Label = NULL;
	GtkWidget *Box = NULL;
	char *str;
	char *act;
	int type;

	type = TYPE_NOTHING;
	if (!attributeset_is_avail(Attr, ATTR_INPUT) &&
		attributeset_is_avail(Attr, ATTR_LABEL))
		type = TYPE_LAB;
	
	if (attributeset_is_avail(Attr, ATTR_INPUT) &&
		!attributeset_is_avail(Attr, ATTR_LABEL))
		type = TYPE_PIX;
	
	if (attributeset_is_avail(Attr, ATTR_INPUT) &&
		attributeset_is_avail(Attr, ATTR_LABEL))
		type = TYPE_LABPIX;

	if (type == TYPE_PIX || type == TYPE_LABPIX) {
		#ifdef USE_GTK2
			Icon = gtk_image_new_from_file(find_pixmap
				(&attributeset_get_first
				(Attr, ATTR_INPUT)[5]));
		#endif
	}
	
	attributeset_set_if_unset(Attr, ATTR_LABEL, "OK");
	attributeset_set_if_unset(Attr, ATTR_DEFAULT, "OK");
	
	switch (type) {
		case TYPE_NOTHING:
		case TYPE_LAB:
			#ifdef USE_GTK2
		    	Button = gtk_button_new_with_label(
			    	attributeset_get_first(Attr, 
					ATTR_LABEL));
			#else
				Box = gtk_hbox_new(FALSE, 5);
		  		Button = gtk_button_new();
	 			Label = gtk_label_new(attributeset_get_first
					(Attr, ATTR_LABEL));
		  		gtk_container_add(GTK_CONTAINER(Button), Box);
		  		gtk_box_pack_end(GTK_BOX(Box), Label, TRUE, TRUE, 3);		  
		  	#endif
			break;
		case TYPE_PIX:
			Button = gtk_button_new();
			#ifdef USE_GTK2 
				gtk_container_add(GTK_CONTAINER(Button), Icon);
			#else
				box1 = xpm_label_box(Button, (find_pixmap
					(&attributeset_get_first
					(Attr, ATTR_INPUT)[5])), "");
				gtk_container_add (GTK_CONTAINER (Button), box1);
			#endif
			break;
		case TYPE_LABPIX:
			Box = gtk_hbox_new(FALSE, 5);
			Button = gtk_button_new();
			Label = gtk_label_new(attributeset_get_first
					(Attr, ATTR_LABEL));
			gtk_container_add(GTK_CONTAINER(Button), Box);
			gtk_box_pack_end(GTK_BOX(Box), Label, TRUE, TRUE, 0);
			#ifdef USE_GTK2 
				gtk_box_pack_end(GTK_BOX(Box), Icon, TRUE, TRUE, 0);
			#else
		  		box1 = xpm_label_box(Box, (find_pixmap
					(&attributeset_get_first
					(Attr, ATTR_INPUT)[5])), "");
          		gtk_container_add (GTK_CONTAINER (Box), box1);
			#endif
	}
	
	/* If a button has no action, the action will be the exit default action. */
	if (!attributeset_is_avail(Attr, ATTR_ACTION)) {
		str = pip_malloc(strlen
			(attributeset_get_first
			(Attr, ATTR_LABEL)) + 10, __func__);
		sprintf(str, "EXIT=\"%s\"\n",
			attributeset_get_first(Attr, ATTR_LABEL));
			attributeset_insert(Attr, ATTR_ACTION, str);
	}

	/* The action attribute can hold arbitrary number of actions, whose execution is a an important task when the button is pressed by the user. */
	act = attributeset_get_first(Attr, ATTR_ACTION);
	while (act != NULL) {
		gtk_signal_connect(GTK_OBJECT(Button),
			"clicked",
			GTK_SIGNAL_FUNC
			(button_pressed), (gpointer) act);
		act = attributeset_get_next(Attr, ATTR_ACTION);
	}

	if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled"))
		gtk_widget_set_sensitive(Button, FALSE);

	return Button;
}

int instruction_execute_push(token Token, AttributeSet * Attr) {
	int Widget_Type;
	static GtkWidget *LastRadioButton = NULL;
	GtkWidget *Widget = NULL;
	#ifdef USE_GTK2
		GtkWidget *OtherWidget;
    #endif
	int n;
	#ifdef EXECUTION_DEBUG
		fprintf(stderr, "%s(): start\n", __func__);
		fflush(stderr);
	#endif
	Widget_Type = Token & WIDGET_TYPE;
	switch (Widget_Type) {
		case WIDGET_LABEL:
			Widget = create_label(Attr);
			push_widget(Widget, Widget_Type);
			break;
		case WIDGET_PIXMAP:
	    	#ifdef USE_GTK2
				Widget = gtk_image_new_from_file(find_pixmap
				    ((&attributeset_get_first
					(Attr, ATTR_INPUT)[5])));
        	#else
          		Widget=gtk_hbox_new(FALSE, 5);
            	box1 = xpm_label_box(Widget, (find_pixmap
					((&attributeset_get_first
					(Attr, ATTR_INPUT)[5]))), "");
            	gtk_container_add (GTK_CONTAINER (Widget), box1);
        	#endif
                    
			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
				gtk_widget_set_sensitive(Widget, FALSE);
			}
			push_widget(Widget, Widget_Type);
			break;
		case WIDGET_ENTRY:
			/// Creating the widget, and pushing it into the stack.
			Widget = gtk_entry_new();
			push_widget(Widget, Widget_Type);

			if (attributeset_is_avail(Attr, ATTR_DEFAULT)) {
				gtk_entry_set_text(GTK_ENTRY(Widget),
				attributeset_get_first(Attr,
				ATTR_DEFAULT));
			}
			
			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "password")) {
			gtk_entry_set_visibility(GTK_ENTRY(Widget), FALSE);
			}
			
			if (attributeset_is_avail(Attr, ATTR_HEIGHT) &&
		    attributeset_is_avail(Attr, ATTR_WIDTH)) {
			gtk_widget_set_usize(Widget,
				atoi(attributeset_get_first	(Attr, ATTR_WIDTH)),
				atoi(attributeset_get_first	(Attr, ATTR_HEIGHT)));
			}
			/// Entries can take actions.
			if (attributeset_is_avail(Attr, ATTR_ACTION)) {
				char *act;
				act = attributeset_get_first(Attr, ATTR_ACTION);
				while (act != NULL) {
					gtk_signal_connect(GTK_OBJECT(Widget),
						"changed",
						GTK_SIGNAL_FUNC
						(entry_inserted),
						(gpointer) act);
					act = attributeset_get_next(Attr, ATTR_ACTION);
				}
			}
			break;
		case WIDGET_TREE:
			Widget = create_tree(Attr);
			push_widget(Widget, Widget_Type);
			break;
		case WIDGET_EDIT:
	    	#ifdef USE_GTK2
	        	Widget = gtk_text_view_new();
            #else
		    	Widget=gtk_text_new(NULL,NULL);
		    	gtk_text_set_editable (GTK_TEXT (Widget), TRUE);
			#endif

			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
				gtk_widget_set_sensitive(Widget, FALSE);
			}
			{
			GtkWidget *Scrolledwindow;

			Scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				(Scrolledwindow),
				GTK_POLICY_AUTOMATIC,
				GTK_POLICY_ALWAYS);

			if (attributeset_is_avail(Attr, ATTR_HEIGHT) &&
			    attributeset_is_avail(Attr, ATTR_WIDTH)) {
				gtk_widget_set_usize(Scrolledwindow,
					atoi (attributeset_get_first (Attr, ATTR_WIDTH)),
					atoi (attributeset_get_first (Attr, ATTR_HEIGHT)));
			} else {
				gtk_widget_set_usize(Scrolledwindow, 200, 100);
			}
			
			gtk_container_add(GTK_CONTAINER(Scrolledwindow), Widget);
			push_widget(Scrolledwindow, WIDGET_SCROLLEDW);
			}	/*command group */

			break;

		case WIDGET_COMBO:
			/// The combo box is quiet simple, however it can be filled with
			/// a GList which contains a list of items.
			Widget = gtk_combo_new();

			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
				gtk_widget_set_sensitive(Widget, FALSE);
			}
			push_widget(Widget, Widget_Type);
			break;

		case WIDGET_OKBUTTON:
			#ifdef USE_GTK2
				Widget = gtk_button_new_from_stock(GTK_STOCK_OK);
			#else
	    	    Widget=gtk_button_new();
        		box1 = xpm_label_box(Widget, find_pixmap(GTK_STOCK_OK), "OK");
         		gtk_container_add (GTK_CONTAINER (Widget), box1);
			#endif
			/// The action attribute can hold arbitrary number of actions,
			/// whose execution is a an important task when the button is
			/// pressed by the user.
			if (attributeset_is_avail(Attr, ATTR_ACTION)) {
				char *act;
				act = attributeset_get_first(Attr, ATTR_ACTION);
				while (act != NULL) {
					gtk_signal_connect(GTK_OBJECT(Widget),
					   "clicked",
					   GTK_SIGNAL_FUNC
					   (button_pressed),
					   (gpointer) act);
				act = attributeset_get_next(Attr, ATTR_ACTION);
				}
			} else {
				gtk_signal_connect(GTK_OBJECT(Widget), "clicked",
					GTK_SIGNAL_FUNC(button_pressed),
					(gpointer) "EXIT=\"OK\"\n");
			}

			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
				gtk_widget_set_sensitive(Widget, FALSE);
			}
			push_widget(Widget, Widget_Type);
			break;

		case WIDGET_CANCELBUTTON:
			#ifdef USE_GTK2
				Widget = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
			#else
	    		Widget=gtk_button_new();
				box1 = xpm_label_box(Widget,
					find_pixmap(GTK_STOCK_CANCEL),
					"Cancel");
        		gtk_container_add (GTK_CONTAINER (Widget), box1);
			#endif
			/// The action attribute can hold arbitrary number of actions,
			/// whose execution is a an important task when the button is
			/// pressed by the user.
			if (attributeset_is_avail(Attr, ATTR_ACTION)) {
				char *act;
				act = attributeset_get_first(Attr, ATTR_ACTION);
				while (act != NULL) {
					gtk_signal_connect(GTK_OBJECT(Widget),
						"clicked",
						GTK_SIGNAL_FUNC
						(button_pressed),
						(gpointer) act);
				act = attributeset_get_next(Attr, ATTR_ACTION);
				}
			} else {
				gtk_signal_connect(GTK_OBJECT(Widget), "clicked",
					GTK_SIGNAL_FUNC(button_pressed),
					(gpointer) "EXIT=\"Cancel\"\n");
			}

			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
				gtk_widget_set_sensitive(Widget, FALSE);
			}
	
			push_widget(Widget, Widget_Type);
			break;

		case WIDGET_HELPBUTTON:
			#ifdef USE_GTK2
				Widget = gtk_button_new_from_stock(GTK_STOCK_HELP);
			#else
				Widget=gtk_button_new();
        		box1 = xpm_label_box(Widget,
        			find_pixmap(GTK_STOCK_HELP), "Help");
        		gtk_container_add (GTK_CONTAINER (Widget), box1);
			#endif

			/// The action attribute can hold arbitrary number of actions,
			/// whose execution is a an important task when the button is
			/// pressed by the user.
			if (attributeset_is_avail(Attr, ATTR_ACTION)) {
				char *act;
				act = attributeset_get_first(Attr, ATTR_ACTION);
				while (act != NULL) {
					gtk_signal_connect(GTK_OBJECT(Widget),
						"clicked",
						GTK_SIGNAL_FUNC
						(button_pressed),
						(gpointer) act);
					act = attributeset_get_next(Attr, ATTR_ACTION);
				}
			} else {
				gtk_signal_connect(GTK_OBJECT(Widget), "clicked",
					GTK_SIGNAL_FUNC(button_pressed),
					(gpointer) "EXIT=\"Help\"\n");
			}

			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
				gtk_widget_set_sensitive(Widget, FALSE);
			}

			push_widget(Widget, Widget_Type);
			break;

		case WIDGET_YESBUTTON:
			#ifdef USE_GTK2
				Widget = gtk_button_new_from_stock(GTK_STOCK_YES);
			#else
				Widget=gtk_button_new();
        		box1 = xpm_label_box(Widget,
        			find_pixmap(GTK_STOCK_YES), "Yes");
		        gtk_container_add (GTK_CONTAINER (Widget), box1);
		
			#endif
			/// The action attribute can hold arbitrary number of actions,
			/// whose execution is a an important task when the button is
			/// pressed by the user.
			if (attributeset_is_avail(Attr, ATTR_ACTION)) {
				char *act;
				act = attributeset_get_first(Attr, ATTR_ACTION);
				while (act != NULL) {
					gtk_signal_connect(GTK_OBJECT(Widget),
						"clicked",
						GTK_SIGNAL_FUNC
						(button_pressed),
						(gpointer) act);
					act = attributeset_get_next(Attr, ATTR_ACTION);
				}
			} else {
				gtk_signal_connect(GTK_OBJECT(Widget), "clicked",
					GTK_SIGNAL_FUNC(button_pressed),
					(gpointer) "EXIT=\"Yes\"\n");
			}

			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
				gtk_widget_set_sensitive(Widget, FALSE);
			}
			push_widget(Widget, Widget_Type);
			break;

		case WIDGET_NOBUTTON:
			#ifdef USE_GTK2
				Widget = gtk_button_new_from_stock(GTK_STOCK_NO);
			#else
				Widget=gtk_button_new();
        		box1 = xpm_label_box(Widget,
        			find_pixmap(GTK_STOCK_NO), "No");
        		gtk_container_add (GTK_CONTAINER (Widget), box1);
			#endif
			/// The action attribute can hold arbitrary number of actions,
			/// whose execution is a an important task when the button is
			/// pressed by the user.
			if (attributeset_is_avail(Attr, ATTR_ACTION)) {
				char *act;
				act = attributeset_get_first(Attr, ATTR_ACTION);
				while (act != NULL) {
					gtk_signal_connect(GTK_OBJECT(Widget),
						"clicked",
						GTK_SIGNAL_FUNC
						(button_pressed),
						(gpointer) act);
					act = attributeset_get_next(Attr, ATTR_ACTION);
				}
			} else {
				gtk_signal_connect(GTK_OBJECT(Widget), "clicked",
					GTK_SIGNAL_FUNC(button_pressed),
					(gpointer) "EXIT=\"No\"\n");
			}

			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
				gtk_widget_set_sensitive(Widget, FALSE);
			}
			push_widget(Widget, Widget_Type);
			break;

		case WIDGET_BUTTON:
			Widget = create_button(Attr);
			push_widget(Widget, Widget_Type);
			break;
		
		case WIDGET_GVIM:
			Widget = create_gvim(Attr);
			push_widget(Widget, Widget_Type);
			break;
	
		case WIDGET_MENUBAR:
			Widget = create_menubar(Attr, pop());
			push_widget(Widget, Widget_Type);
			break;
	
		case WIDGET_MENU:
			attributeset_set_if_unset(Attr, ATTR_LABEL, "Unlabelled menu");
			Widget = create_menu(Attr, pop());
			push_widget(Widget, Widget_Type);
			break;
	
		case WIDGET_MENUITEM:
			attributeset_set_if_unset(Attr, ATTR_LABEL, "Unlabelled menu item");
			Widget = create_menuitem(Attr);
			push_widget(Widget, Widget_Type);
			break;
		
		case WIDGET_CHECKBOX:
			attributeset_set_if_unset(Attr, ATTR_LABEL,
				"Uninitialized checkbox");
			Widget = 
				gtk_check_button_new_with_label(attributeset_get_first
						    (Attr, ATTR_LABEL));
			
			if (attributeset_cmp_left(Attr, ATTR_DEFAULT, "true") ||
		    attributeset_cmp_left(Attr, ATTR_DEFAULT, "yes")) {
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
						     (Widget), TRUE);
			}
			
			/// Checkboxes also can take actions...
			if (attributeset_is_avail(Attr, ATTR_ACTION)) {
				char *act;
				act = attributeset_get_first(Attr, ATTR_ACTION);
				while (act != NULL) {
					gtk_signal_connect(GTK_OBJECT(Widget),
							"toggled",
							GTK_SIGNAL_FUNC
							(button_toggled),
							(gpointer) act);
				act = attributeset_get_next(Attr, ATTR_ACTION);
				}
			}

			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
				gtk_widget_set_sensitive(Widget, FALSE);
			}
			push_widget(Widget, Widget_Type);
			break;

		case WIDGET_RADIO:
			attributeset_set_if_unset(Attr, ATTR_LABEL, "Unnamed");
			/// It is a little strange hack with the radiobuttons.
			/// Not easy to force them work together.
			if (LastRadioButton == NULL) {
				Widget = gtk_radio_button_new_with_label(NULL,
					attributeset_get_first
					(Attr, ATTR_LABEL));
				LastRadioButton = Widget;
			} else {
				Widget =
			    	gtk_radio_button_new_with_label_from_widget
			    	(GTK_RADIO_BUTTON(LastRadioButton),
			     	attributeset_get_first(Attr, ATTR_LABEL));
			}
		
			/// Radiobuttons also can take actions...
			if (attributeset_is_avail(Attr, ATTR_ACTION)) {
				char *act;
				act = attributeset_get_first(Attr, ATTR_ACTION);
				while (act != NULL) {
					gtk_signal_connect(GTK_OBJECT(Widget),
						"toggled",
						GTK_SIGNAL_FUNC
						(button_toggled),
						(gpointer) act);
				act = attributeset_get_next(Attr, ATTR_ACTION);
				}
			}

			push_widget(Widget, Widget_Type);

			if (attributeset_cmp_left(Attr, ATTR_VISIBLE, "disabled")) {
			gtk_widget_set_sensitive(Widget, FALSE);
			}
			break;

		case WIDGET_LIST:
			Widget = gtk_list_new();
      		/// When we have create a new LIST widget, we also have to make
      		/// GtkScrolledWindow, because this is the only way we can 
      		/// scroll in the list. We put the _scrolled window_ into the 
      		/// stack, because this holds the list.
			{
			GtkWidget *Scrolledwindow;
			Scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
			if (attributeset_is_avail(Attr, ATTR_HEIGHT)
				&& attributeset_is_avail(Attr, ATTR_WIDTH)) {
				gtk_widget_set_usize(Scrolledwindow,
					atoi (attributeset_get_first (Attr, ATTR_WIDTH)),
					atoi (attributeset_get_first (Attr, ATTR_HEIGHT)));
			} else {
				gtk_widget_set_usize(Scrolledwindow, 300, 100);
			}
			gtk_scrolled_window_add_with_viewport
				(GTK_SCROLLED_WINDOW(Scrolledwindow), Widget);

			push_widget(Scrolledwindow, WIDGET_SCROLLEDW);
			}
			
			/// The LIST widget can handle actions...
			if (attributeset_is_avail(Attr, ATTR_ACTION)) {
				char *act;
				act = attributeset_get_first(Attr, ATTR_ACTION);
				while (act != NULL) {
					gtk_signal_connect(GTK_OBJECT(Widget),
						"selection_changed",
						GTK_SIGNAL_FUNC
						(list_selection_changed),
						(gpointer) act);
					act = attributeset_get_next(Attr, ATTR_ACTION);
				}
			}
			break;

		case WIDGET_TABLE:
			{
			list_t *table_header;
			//int columns;
			if (attributeset_is_avail(Attr, ATTR_LABEL)) {
				table_header =
				    linecutter(attributeset_get_first
					(Attr, ATTR_LABEL), '|');
				Widget =
					gtk_clist_new_with_titles
				    (table_header->n_lines,
				    table_header->line);
			} else {
				Widget = gtk_clist_new(1);
			}
			}
			/// We need connect the select_row to a function to store the
			/// selected row.
			gtk_signal_connect(GTK_OBJECT(Widget), "select_row",
				(GtkSignalFunc)(table_selection_made),
				strdup(attributeset_get_first
				(Attr, ATTR_VARIABLE)));
		
			{
			char *act;
			act = attributeset_get_first(Attr, ATTR_ACTION);
			while (act != NULL) {
				gtk_signal_connect(GTK_OBJECT(Widget),
					"select_row",
					(GtkSignalFunc)
					(table_selected),
					strdup(act));
				act = attributeset_get_next(Attr, ATTR_ACTION);
			}
			}

			/// When create a new TABLE (clist in gtk) widget, we 
			/// also have to create a GtkScrolledWindow, because this is the
			/// only way we can scroll in the clist. We put the _scrolled 
			/// window_ into the stack, because this holds the list.
			{
			GtkWidget *Scrolledwindow;

			gtk_clist_set_shadow_type(GTK_CLIST(Widget), GTK_SHADOW_OUT);
			Scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW
				(Scrolledwindow),
				GTK_POLICY_AUTOMATIC,
				GTK_POLICY_ALWAYS);

			if (attributeset_is_avail(Attr, ATTR_HEIGHT) &&
			    attributeset_is_avail(Attr, ATTR_WIDTH)) {
				gtk_widget_set_usize(Scrolledwindow,
						     atoi
						     (attributeset_get_first
						      (Attr, ATTR_WIDTH)),
						     atoi
						     (attributeset_get_first
						      (Attr, ATTR_HEIGHT)));
			} else {
				gtk_widget_set_usize(Scrolledwindow, 200, 100);
			}

			gtk_container_add(GTK_CONTAINER(Scrolledwindow), Widget);

			push_widget(Scrolledwindow, WIDGET_SCROLLEDW);
			}	/*command group */

			break;
			
		case WIDGET_VBOX:
			{
			/// Creating a box is not a simple process, because we have to
			/// add the widgets to it. First we remove the widgets from the
			/// top of the stack (only one stack element with more widgets,
			/// thanks to the SUM instruction), then we add it to the box.
			/// The last step is pushing the box back to the stack.
			stackelement s;
			s = pop();
			Widget = gtk_vbox_new(FALSE, 5);
			for (n = 0; n < s.nwidgets; ++n)
				if (s.widgettypes[n] == WIDGET_EDIT ||
				    s.widgettypes[n] == WIDGET_FRAME ||
				    s.widgettypes[n] == WIDGET_SCROLLEDW) {
					gtk_box_pack_start(GTK_BOX(Widget),
						s.widgets[n],
						TRUE, TRUE, 0);
				} else {
					gtk_box_pack_start(GTK_BOX(Widget),
						s.widgets[n],
						FALSE, FALSE, 0);
				}
			push_widget(Widget, Widget_Type);
			/// The box widgets are holding the radiobuttons to groups, so
			/// one radiobutton can turn off the others. 

			LastRadioButton = NULL;
			}	/*command group */
			break;
	
		case WIDGET_HBOX:
			{
			/// The hbox is very similar to vbox...
			stackelement s;
			s = pop();
			Widget = gtk_hbox_new(FALSE, 5);
			for (n = s.nwidgets - 1; n >= 0; --n)
				if (s.widgettypes[n] == WIDGET_EDIT ||
				    s.widgettypes[n] == WIDGET_FRAME ||
				    s.widgettypes[n] == WIDGET_SCROLLEDW ||
				    s.widgettypes[n] == WIDGET_ENTRY) {
					gtk_box_pack_end(GTK_BOX(Widget),
						s.widgets[n],
						TRUE, TRUE, 0);
				} else {
					gtk_box_pack_end(GTK_BOX(Widget),
						s.widgets[n],
						FALSE, FALSE, 0);
				}
			push_widget(Widget, Widget_Type);
			/// The box widgets holds the radiobuttons to groups, so
			/// one radiobutton can turn off the others. 
			LastRadioButton = NULL;
			}
			break;

		case WIDGET_FRAME:
			{
			stackelement s;
			GtkWidget *vbox;
			s = pop();
			vbox = gtk_vbox_new(FALSE, 5);
			gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);

			for (n = 0; n < s.nwidgets; ++n)
				if (s.widgettypes[n] == WIDGET_EDIT ||
				    s.widgettypes[n] == WIDGET_FRAME ||
				    s.widgettypes[n] == WIDGET_SCROLLEDW) {
					gtk_box_pack_start(GTK_BOX(vbox),
						s.widgets[n],
						TRUE, TRUE, 0);
				} else {
					gtk_box_pack_start(GTK_BOX(vbox),
						s.widgets[n],
						FALSE, FALSE, 0);
				}
			
			if (attributeset_is_avail(Attr, ATTR_LABEL)) {
				Widget = gtk_frame_new(attributeset_get_first
					(Attr, ATTR_LABEL));
			} else {
				Widget = gtk_frame_new(NULL);
			}
			gtk_container_add(GTK_CONTAINER(Widget), vbox);
			}
			push_widget(Widget, Widget_Type);
			/// The frame widget holds the radiobuttons to groups, so
			/// one radiobutton can turn off the others.
			LastRadioButton = NULL;
			break;

		default:
			fprintf(stderr, "%s(): Unknown widget.\n", __func__);
			fflush(stderr);
	}	/* end switch (Widget_Type) */
	#ifdef EXECUTION_DEBUG
		fprintf(stderr, "%s(): Widget created.\n", __func__);
		fflush(stderr);
	#endif
	/// Now we create a new variable or refresh the old one for this
	/// new widget.
	variables_new_with_widget(Attr, Widget, Widget_Type);
	variables_set_attributes(attributeset_get_first
		(Attr, ATTR_VARIABLE), Attr);
	variables_set_parent(attributeset_get_first(Attr, ATTR_VARIABLE),
		window);
	#ifdef EXECUTION_DEBUG
		fprintf(stderr, "%s(): Variable created.\n", __func__);
		fflush(stderr);
	#endif

	return 0;
}


/// The next few functions operates on stack

stackelement _sum(stackelement a, stackelement b) {
	int n;
	/* Let's copy the widgets from b to a */
	for (n = 0; n < b.nwidgets; ++n) {
		a.widgets[a.nwidgets + n] = b.widgets[n];
		a.widgettypes[a.nwidgets + n] = b.widgettypes[n];
	}
	a.nwidgets += b.nwidgets;
	return (a);
}

void _more_memory_needed(void) {
	void *memory = NULL;

	if (program == NULL) {
		memory_counter = 128;
		memory = malloc(memory_counter * sizeof(instruction));
	} else {
		memory_counter *= 2;
		memory =
		    realloc(program, memory_counter * sizeof(instruction));
	}

	if (memory == NULL) {
		fprintf(stderr, "%s: not enought memory.\n", __func__);
		exit(EXIT_FAILURE);
	}
	#ifdef DEBUG
		fprintf(stderr, "%s: Memory allocated: %d.\n",
			__func__, memory_counter);
	#endif

	program = memory;
}

void instruction_new(instruction new) {
	#ifdef EXECUTION_DEBUG
		fprintf(stderr,
			"%s(): instruction_counter=%d, memory_counter=%d\n",
			__func__, instruction_counter, memory_counter);
		fflush(stderr);
	#endif
	if (instruction_counter == memory_counter)
		_more_memory_needed();

	program[instruction_counter++] = new;
}

/// Functions to store a token in the memory

int token_store(token command) {
	instruction inst;
	inst.command = command;
	inst.argument = NULL;
	instruction_new(inst);
	return (0);
}

int token_store_with_argument(token command, const char *argument) {
	instruction inst;
	int sub_attribute;

	#ifdef EXECUTION_DEBUG
		if (argument == NULL) {
			fprintf(stderr, "%s(): argument is NULL.\n", __func__);
			fflush(stderr);
		}
	#endif
	if (argument == NULL)
		argument = "";

	#ifdef EXECUTION_DEBUG
		printf("%s(): Command: %08x '%s'\n", __func__, command, argument);
	#endif

	sub_attribute = (command & SUB_ATTRIBUTE) >> 24;
	inst.argument = pip_malloc(strlen(argument) + 32, __func__);
	switch (sub_attribute) {
		case 0:
			/// No sub_attribute, it is just a normal text.
			inst.argument = strdup(argument);
			sprintf(inst.argument, "%s", argument);
			break;
	
		case 1:
			/// The text argument is a shell command.
			sprintf(inst.argument, "Command:%s", argument);
			break;
	
		case 2:
			/// The text argument is a file name.
			sprintf(inst.argument, "File:%s", argument);
			break;
	
		case 3:
			/// The text argument is a widget name which must be
			/// cleared.
			sprintf(inst.argument, "Clear:%s", argument);
			break;
	
		case 4:
			sprintf(inst.argument, "Append:%s", argument);
			break;
	
		case 10:
			/// The text argument is widget name which must be filled with
			/// a file name from a file selection dialog.
			sprintf(inst.argument, "Fileselect:%s", argument);
			break;
	
		case 11:
			/// The text argument is widget name and we have to remove the
			/// selected element from it.
			sprintf(inst.argument, "Removeselected:%s", argument);
			break;
	
		case 12:
			/// The text argument is widget name and we have to remove the
			/// selected element from it.
			sprintf(inst.argument, "Refresh:%s", argument);
			break;
	
		case 13:
			/// The text argument is widget name and we have to
			/// remove the selected element from it.
			sprintf(inst.argument, "Launch:%s", argument);
			break;
	
		default:
			fprintf(stderr, "%s(): Warning: Unknown sub-attribute %d",
				__func__, sub_attribute);
			fflush(stderr);
		}
	inst.command = command;
	instruction_new(inst);
	return (0);
}
