
/*actions.h*/
#ifndef ACTIONS_H
#define ACTIONS_H

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

//#include "stack.h"
//#include "attributes.h"
//#include "stringman.h"
//#include "automaton.h"
//#include "main.h"
//#include "variables.h"	//GN

typedef int token;

typedef struct instruction {
  token        command;
  char         *argument;
} instruction;

typedef struct actioncommand {
  GtkWidget   *source_widget;
  char        *destination_name;
} actioncommand;

/* The elements in stack can contain MAXWIDGETS number of widgets. */
typedef struct stackelement {
  char       *cval;
  int         nwidgets;
  GtkWidget   *widgets[128];
  int         widgettypes[128];
} stackelement;



extern char *program_name;
extern char *includefile;

/*
** Functions defined in actions.c
*/
int action_closewindow( GtkWidget *widget, char *string );
int action_launchwindow( GtkWidget *widget, char *string );
int action_exitprogram( GtkWidget *widget, char *string );
int action_refreshwidget( GtkWidget *widget, char *string );
int action_fileselect( GtkWidget *widget, char *string );
void action_fileselection_made( GtkWidget *w, actioncommand *ac );
void action_fileselection_destroy( GtkWidget *w, actioncommand *ac );
int action_clearwidget( GtkWidget *widget, char *string );
int action_removeselected( GtkWidget *widget, char *string );
int action_append( GtkWidget *widget, char *string );
int action_shellcommand( GtkWidget *widget, char *string );
int action_enable(GtkWidget * widget, char *string);	//GN
int action_disable(GtkWidget * widget, char *string);	//GN
int action_savewidget(GtkWidget * widget, char *string);	//GN
int instruction_execute(instruction command);	//GN

#endif


/*attributes.h*/
#ifndef ATTRIBUTES_H
#define ATTRIBUTES_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
//#include "memory.h"

#define ATTRNUMBER  32

typedef struct AttributeSet {
  int    n_attr;      /* How many attributes can be found here?  */
  GList  **attr;      /* Attributes are linked lists.            */
  GList  **_pointer;  /* Pointers for incremental read function. */
} AttributeSet;


AttributeSet *attributeset_new( void );
gboolean attributeset_is_avail( AttributeSet *set, int attribute );
gboolean attributeset_cmp_left( AttributeSet *set, int attribute,
                                const char *str );
char *attributeset_get_first( AttributeSet *set, int attribute );
char *attributeset_get_next( AttributeSet *set, int attribute );
GList *attributeset_get_whole_list( AttributeSet *set, int attribute );
char *attributeset_set_if_unset( AttributeSet *set, int attribute,
                            const char *s);
char *attributeset_insert( AttributeSet *set, int attribute, 
                            const char *s);
int attributeset_clean( AttributeSet *set );
#endif

/*automaton.h*/
#ifndef AUTOMATON_H
#define AUTOMATON_H

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

//#include "gtkdialog.h"
//#include "stack.h"
//#include "attributes.h"
//#include "stringman.h"
//#include "main.h"

extern char   *program_src;
extern int    source;
extern int    charsreaded;
extern FILE   *sourcefile;
extern int    debug;
char *wtitle;




/*                                -- Type of widget      */
/*                              xxxxxxxx                 */
#define WIDGET_TYPE           0x00FF0000
#define WIDGET_LABEL          0x00010000
#define WIDGET_ENTRY          0x00020000
#define WIDGET_EDIT           0x00030000
#define WIDGET_BUTTON         0x00040000
#define WIDGET_CHECKBOX       0x00050000
#define WIDGET_RADIO          0x00060000
#define WIDGET_LIST           0x00070000
#define WIDGET_TABLE          0x00080000
#define WIDGET_COMBO          0x00090000
#define WIDGET_OKBUTTON       0x000A0000
#define WIDGET_CANCELBUTTON   0x000B0000
#define WIDGET_HELPBUTTON     0x000C0000
#define WIDGET_NOBUTTON       0x000D0000
#define WIDGET_YESBUTTON      0x000E0000
#define WIDGET_SCROLLEDW      0x000F0000
#define WIDGET_VBOX           0x00100000
#define WIDGET_HBOX           0x00200000
#define WIDGET_FRAME          0x00300000
#define WIDGET_WINDOW         0x00400000
#define WIDGET_PIXMAP         0x00500000
#define WIDGET_MENUBAR        0x00600000
#define WIDGET_MENU           0x00700000
#define WIDGET_MENUITEM       0x00800000
#define WIDGET_GVIM           0x00900000
#define WIDGET_TREE           0x00A00000
/*                                   - Widget subtype    */
/*                              xxxxxxxx                 */
#define WIDGET_SUBTYPE        0x00000F00
/*                                  - Command            */
/*                              xxxxxxxx                 */
#define COMMAND               0x0000F000
#define PUSH                  0x00001000
#define SUM                   0x00002000
#define SET                   0x00003000
/*                                    -- Attribute           */
/*                              xxxxxxxx                 */
#define ATTRIBUTE             0x000000ff
#define ATTR_LABEL            0x00000001
#define ATTR_DEFAULT          0x00000002
#define ATTR_VARIABLE         0x00000003
#define ATTR_HEIGHT           0x00000004
#define ATTR_WIDTH            0x00000005
#define ATTR_INPUT            0x00000006
#define ATTR_VISIBLE          0x00000007
#define ATTR_ACTION           0x00000008
#define ATTR_ITEM             0x00000009
#define ATTR_OUTPUT           0x0000000A
#define ATTR_SOCKET           0x0000000B
/*                               - Sub attribute         */
/*                              xxxxxxxx                 */
#define SUB_ATTRIBUTE         0x0F000000
#define SUB_ATTR_SHELL        0x01000000
#define SUB_ATTR_FILE         0x02000000
#define SUB_ATTR_CLEAR        0x03000000
#define SUB_ATTR_APPEND       0x04000000
#define SUB_ATTR_FILESEL      0x0A000000
#define SUB_ATTR_REMOVE       0x0B000000
#define SUB_ATTR_REFRESH      0x0C000000
#define SUB_ATTR_LAUNCH       0x0D000000

/*
** Functions in the automaton.c
*/
stackelement _sum( stackelement a, stackelement b);
void fileselection_made( GtkWidget *w, actioncommand *action );

//GN void fill_text_by_file( GtkTextBuffer *buffer, char *filename );
void fill_list_by_glist( GtkWidget *list, GList *glist );
void fill_clist_by_glist( GtkWidget *list, int col, GList *glist );
void fill_list_by_command( GtkWidget *list, char *command );
void fill_entry_by_command( GtkWidget *entry, char *command);
void fill_table_by_command( GtkWidget *list, char *command );
void window_title(char *string);
void button_pressed(GtkWidget *, gpointer);
int instruction_execute_push(token Token, AttributeSet * Attr);	//GN
int token_store(token command);	//GN
int token_store_with_argument(token command, const char *argument);	//GN
void run_program();	//GN
#endif


/*gtkdialog.h*/
#ifndef GTKDIALOG_H
#define GTKDIALOG_H

#undef DEBUG
#undef MEMORY_DEBUG
#undef EXECUTION_DEBUG

#endif


/*main.h*/
#include "config.h"
#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>
#include <gtk/gtk.h>
//#include "variables.h"
//#include "automaton.h"


#define PRG_UNKNOWN 0
#define PRG_MEMORY  1
#define PRG_STDIN   2
#define PRG_FILE    3

int print_help_exit( int exitcode );	//GN
int print_version_exit( int exitcode );	//GN
int get_program_from_variable( char *name ); 	//GN
int get_program_from_file( char *name );	//GN
int getnextchar(void); //GN

/*memory.h*/

#ifndef MEMORY_H
#define MEMORY_H

#include <stdio.h>
#include <stdlib.h>
//#include "gtkdialog.h"
#include <gtk/gtk.h>

void *pip_malloc( size_t size, const char *caller );
void pip_free(void *memory, const char *caller);

#endif


/*stack.h*/
#ifndef MYSTACK_H
#define MYSTACK_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#define MAXWIDGETS 128;




stackelement pop();
void push(stackelement a);
void push_widget(GtkWidget * widget, int widgettype); //GN

#endif

/*stringman.h*/
#ifndef STRINGMAN_H
#define STRINGMAN_H

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef struct list_t {
	int n_lines;
	int maxlines;
	char **line;
} list_t;

list_t *linecutter(const char *str, int fs );
char *find_pixmap( char *filename );
char *str_default_name(int itype);	//GN

#endif

/*variables.h*/
#ifndef VARIABLES_H
#define VARIABLES_H

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
//#include "attributes.h"
//#include "automaton.h"

#define NAMELEN 512

typedef struct variable {
  char         Name[ NAMELEN+1 ]; /* The name of the variable   */
  GtkWidget    *Widget;           /* The pointer to the widget  */
  int          Type;              /* The type of the widget     */
  AttributeSet *Attributes;       /* Attributes for Widget      */
  GtkWidget    *ParentWindow;     /* Parent Window of Widget    */
  int          row, column;       /* Selected items position.   */
  int          ncolumns;          /* How many columns is in it. */
  struct variable *left;          /* The pointer used in to or- */
  struct variable *right;         /* ganise the Btree           */
  int autonamed;		  /* have a default name?       */
} variable;

/*
** Functions:
*/
variable *variables_new( const char *name );
variable *variables_new_with_widget(AttributeSet *Attr,
				    GtkWidget * widget, int type);

gboolean variables_is_avail_by_name( const char *name );
variable *variables_get_by_name( const char *name );

variable *variables_set_widget( const char *name, GtkWidget *widget );
variable *variables_set_parent( const char *name, GtkWidget *parent );
variable *variables_set_type( const char *name, int type );
variable *variables_set_attributes( const char *name, AttributeSet *a);
variable *variables_set_value( const char *name, char *value );
variable *variables_set_row_column( const char *name,
                                    int row, int column );


variable *variables_refresh( const char *name );
variable *variables_clear( const char *name );
int remove_selected_variable(const char *name);
int append_fromto_variable(const char *from, const char *to);


void variables_export_all( void );
void variables_initialize_all( void );

void variables_print_debug( variable *actual );
void variables_print_one( variable *var );
void variables_drop_by_parent(variable * actual, GtkWidget * Parent); //GN
void print_variables(variable * actual);	//GN
variable *variables_save(const char *name); //GN
variable *variables_enable(const char *name);	//GN
variable *variables_disable(const char *name); //GN
variable *_tree_find(const char *name, variable * actual); //GN
int _tree_insert(variable * new, variable * actual); //GN
#endif

/*widgets.h*/
#ifndef WIDGETS_H
#define WIDGETS_H

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include <gtk/gtk.h>
//#include "stack.h"
//#include "attributes.h"
//#include "stringman.h"
//#include "variables.h"
//#include "automaton.h"

char *widget_get_text_value( GtkWidget *widget, int type );
int widget_label_refresh( variable *var );
int widget_edit_refresh( variable *var );
void widget_entry_refresh( variable *var );
void widget_list_refresh( variable *var );
void widget_table_refresh( variable *var );
void widget_combo_refresh( variable *var );
FILE *widget_opencommand( const char *command );
void save_edit_to_file(GtkWidget * widget, char *filename);

//GN void fill_text_by_file( GtkTextBuffer *buffer, char *filename );
void fill_list_by_glist( GtkWidget *list, GList *glist );
void fill_clist_by_glist( GtkWidget *list, int columns, GList *glist);
void fill_clist_by_command( GtkWidget *list,int columns,char *command );
void fill_list_by_command( GtkWidget *list, char *command );
void fill_entry_by_command( GtkWidget *entry, char *command);
void fill_table_by_command( GtkWidget *list, char *command );
char *widgets_to_str( int itype );
void widget_edit_save(variable * var);	//GN

#endif


