/* 
** main.c: Starting up the program.
** copyright: (c) 2003 by László Pere
** email: pipas@linux.pte.hu
**
** This program is free software; you can redistribute it and/or 
** modify  it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of
** the License, or (at your option) any later version.
**
** $Id: main.c,v 1.1 2004/11/16 20:47:40 root Exp root $
** $Log: main.c,v $
** Revision 1.1  2004/11/16 20:47:40  root
** Initial revision
*/

#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>
#include <getopt.h>
#include <gtk/gtk.h>
#include "main.h"
#include "variables.h"
#include "automaton.h"
#include "config.h"
#include "parser.h"

char *program_src;
char *program_name;

char *includefile = NULL;
int source = PRG_UNKNOWN;
int charsreaded = 0;
FILE *sourcefile = NULL;
int debug = 0;

static struct option const long_options[] = {
	{ "help", no_argument, 0, 'h' },
	{ "version", no_argument, 0, 'v' },
	{ "program", required_argument, 0, 'p' },
	{ "filename", required_argument, 0, 'f' },
	{ "include", required_argument, 0, 'i' },
	{ "debug", no_argument, 0, 'd' },
	{ "stdin", no_argument, 0, 's' },
	{ NULL, 0, NULL, 0 }
};

int first_free_argument;

void gtkdialog_init( int argc, char *argv[] )
{
	int c;
	while ((c=getopt_long(argc, argv, "hvp:f:i:ds", 
					long_options, NULL)) != -1 ) {
		switch (c) {
			case 'h':
				print_help_exit( EXIT_SUCCESS );
				break;
			case 'v':
				print_version_exit( EXIT_SUCCESS );
				break;
			case 'd':
				debug = 1;
				break;
			case 'i':
				includefile = optarg;
				break;
			case 's':	
				program_name = "STDIN";
				source = PRG_STDIN;
				break;
			case 'p':
				get_program_from_variable(optarg);
				break;
			case 'f':
				get_program_from_file(optarg);
				break;
			default:
				exit( EXIT_FAILURE );
		}
	}
	first_free_argument = optind;
}

int get_program_from_variable( char *name )
{
	program_name = name;

	program_src = getenv(program_name);
	if (!program_src) {
		fprintf(stderr,
		"Could not find environment variable '%s'.\n",
			program_name);
		exit(EXIT_FAILURE);
	}
	source = PRG_MEMORY;
	return 0;	//GN
}
	
int get_program_from_file( char *name )
{
	int tmp;
	
	program_name = "MAIN_WINDOW";
	sourcefile = fopen(name, "r");
	if (!sourcefile) {
		fprintf(stderr, "Error: %m\n" );
		exit(EXIT_FAILURE);
	}
	/* FIXME: We read the first line of file and drop it. It is
	 * required because the #! is not included in the language.
	 */
	program_src = NULL;
	
	getline(&program_src, &tmp, sourcefile);
	free(program_src);
	program_src = NULL;
	source = PRG_FILE;
	return 0;	//GN
}

int print_help_exit( int exitcode )
{
	fprintf(stderr,
"%s" "Usage: gtdkdialog [options]\n"
"-h, --help                Print usage information and exit.\n"
"-v, --version             Print version information and exit.\n"
"-p, --program VARNAME     Get program from an environment  \n"
"                          variable (just the name needed.  \n"
" -s, --stdin              Read program from the stdin.     \n"
" -f [filename]            Get program from file.           \n"
" -i <filename>            Include file as bash script.     \n"
"                          #! /usr/local/bin/gtkdialog -f   \n"
"                          as the first line.               \n"
" -d | --debug             Debug mode.                      \n",PACKAGE_NAME);
	exit( exitcode );
}

int print_version_exit( int exitcode )
{	
	printf("%s version %s\n", PACKAGE_NAME, PACKAGE_VERSION);
	exit( exitcode );
}


int getnextchar(void)
{
	int c=0;
	if (source == PRG_MEMORY) {
		if (program_src == NULL
		    || charsreaded > strlen(program_src))
			return (EOF);
		c = (int) program_src[charsreaded];
		if (c == '\0')
			c = EOF;
		++charsreaded;
	}
	if (source == PRG_STDIN) {
		c = (int) getchar();
	}
	if (source == PRG_FILE) {
		c = (int) getc(sourcefile);
	}
	if (debug != 0)
		printf("%c", c);

	return (c);
}

int main(int argc, char *argv[])
{
#ifdef DEBUG
	fprintf(stderr, "%s: Starting.\n", __func__);
	fflush(stderr);
#endif
	setlocale(LC_ALL, "");
	gtk_init(&argc, &argv);
	gtkdialog_init(argc, argv);
	yyparse();
	return 0;	//GN
}
